/*
     Verify that this user/password is valid per a database.
     Return 1 if verified, 0 otherwise.
     
     Format of connection string (look like internet URL):

       db://user:password@hostname/database/table?name&passwd

     Example connect to Oracle RDBMS at user 'roger' with password
     'tiger', to 'host.domain.com' database server, fields name in
     table is 'name' and 'passwd' in 'oshadow' table:

       oracle://roger:tiger@host.domain.com/tacacs/oshadow?name&passwd

     DONE:
     12-nov-1998 Created
                 Add DB support to 'login = db <string>'
     14-nov-1998 Change Tacacs+ version from 0.95 to 3.0.9
     18-nov-1998 Added code for Oracle [version 8.0.5]
     27-nov-1998 Tested with 30'000 usernames Oracle database
                 Added DB support to global configuration
                 'default authentication = db <string>'
     28-nov-1998 Added code for NULL database %)
     
     FUTURE:
     Make *_db_verify() the functions is reenterable
     More security for connection to database
     GDBM support
     Separate debug logging
     Perfomance testing on 10000 records in Oracle database
     (in guide sayd about 3 auth/sec on Ultra 2 - hmm)
     
     -------------------------------------------------------
     fil@artelecom.ru                   http://twister.pp.ru
 
 ****************************************************************************
				    PART II

   I am added some extra extension. Like MySQL and PostgreSQL database support
   And change most of lines for use dynamic memory allocation. db_accounting 
   added by me.
  
   devrim(devrim@gazi.edu.tr)

 ****************************************************************************
				    PART III

   I have change the way how the DB flows to address features such as recursive
   lookups for users and groups.  Plus, I added and changed a few things.

   August 20, 2002 - Added function (parse_db) and structure (dbinfo) to
   		     minimize the amount of parameters that are sent to the
		     other functions.

   August 22, 2002 - Changed db_acct to allow for more information to be
                     inserted into the database

   August 22, 2002 - Changed db_acct to create the SQL statement then send
   		     it the supporting functions.  All databases use in this
		     program support SQL except for db_null.

   baram01@hotmail.com
*/

#include "tac_plus.h"

#if defined(USE_DB)
#include <stdio.h>

#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#include "db.h"

/* For checking DB type */
int 
check_db_type( char *db_type )
{
char *dbp[]=DEFINED_DB;
int ret=1,i;

for (i=0; dbp[i] ; i++ ) {
	if(!strcmp(db_type,dbp[i])) {
		ret=0;
		break;
	}
}
return ret;
}

TAC_DBINFO *
db_parse_url(char *url)
{
	char *pUrl;
	void *pAlloc;
	TAC_DBINFO *db_info;
	int	len;

	if (!url) {
		report(LOG_ERR,"db_parse_url: called with NULL url");
		return(NULL);
	}

	if (debug & DEBUG_PARSE_FLAG) {
		report(LOG_DEBUG, "db_parse_url: %s", url);
	}

	len = strlen(url) + sizeof(TAC_DBINFO) + 1;

	len *= sizeof(char);	/* to make sure there is enough room */

	pAlloc = tac_malloc(len);  /* don't need to check return because tac_malloc does */
	bzero(pAlloc, len);

	db_info = (TAC_DBINFO *)pAlloc;
	pUrl = pAlloc;
	pUrl += sizeof(TAC_DBINFO);	  /* put the copy of the string after the structure */

	strcpy(pUrl, url);
	
	db_info->type = pUrl;
	
    if( NULL == (pUrl = (char *)strstr( pUrl, "://" )) ) {
		report(LOG_ERR,"db_parse_url: parsing user");
		free(pAlloc);
		return(NULL);
	}
	*pUrl = '\0'; 
	pUrl += 3;

	/* For recognize db authentication database */
    
    if (check_db_type(db_info->type)) {
		report(LOG_ERR,"db_parse_url: %s DB authentication scheme " \
			   "not recognized by tac_plus",db_info->type);
		free(pAlloc);
		return(NULL);
	}

	db_info->user = pUrl;

	if( NULL == (pUrl = (char *)strchr( pUrl, ':' )) ) {
		report(LOG_ERR,"db_parse_url: parsing password");
		free(pAlloc);
		return(NULL);
	}
	*(pUrl++) = '\0';
	db_info->password = pUrl;
    
    if( NULL == (pUrl = (char *)strchr( pUrl, '@' )) ) {
		report(LOG_ERR,"db_parse_url: parsing hostname");
		free(pAlloc);
		return(NULL);
    }
	*(pUrl++) = '\0';
	db_info->host = pUrl;
    
	if( NULL == (pUrl = (char *)strchr( pUrl, '/' )) ) {
		report(LOG_ERR,"db_parse_url: parsing database name");
		free(pAlloc);
		return(NULL);
	} 
	*(pUrl++) = '\0';
	db_info->database = pUrl;
    
	if( NULL == (pUrl = (char *)strchr( pUrl, '/' )) ) {
		db_info->table = DEFAULT_DB_TABLE_NAME;
		db_info->field_name = DEFAULT_DB_FIELD_NAME_USER;
		db_info->field_passwd = DEFAULT_DB_FIELD_NAME_PASSWD;
		if (debug & DEBUG_PARSE_FLAG)
			report(LOG_INFO,"db_parse_url: Using default table");
	} else { 
		*(pUrl++) = '\0';
		db_info->table = pUrl;

		if( NULL == (pUrl = (char *)strchr( pUrl, '?' )) ) {
			db_info->field_name = DEFAULT_DB_FIELD_NAME_USER;
			db_info->field_passwd = DEFAULT_DB_FIELD_NAME_PASSWD;
			if (debug & DEBUG_PARSE_FLAG)
				report(LOG_DEBUG, "Using default field names: " \
					   "userid: %s, password: %s",
					   db_info->field_name, db_info->field_passwd);
		} else {
			*(pUrl++) = '\0';
			db_info->field_name = pUrl;

			if( NULL == (pUrl = (char *)strchr( pUrl, '&' )) ) {
				db_info->field_passwd = DEFAULT_DB_FIELD_NAME_PASSWD;
				if (debug & DEBUG_PARSE_FLAG)
					report(LOG_DEBUG, "Using default password field name %s",
						   db_info->field_passwd );
			} else {
				*(pUrl++) = '\0';
				db_info->field_passwd = pUrl;
			}
		}
	}

    /* Parse database connection string */
	if (debug & DEBUG_PARSE_FLAG)
		report(LOG_DEBUG, "db_parse_url: db_type=%s, db_user=%s, " \
			   "db_password=%s db_hostname=%s, db_name=%s, " \
			   "db_table=%s, dbfield_name=%s, dbfield_passwd=%s", 
			   db_info->type, db_info->user, db_info->password, 
			   db_info->host, db_info->database, db_info->table, 
			   db_info->field_name, db_info->field_passwd);

	return (db_info);
}

int db_connect(DBH *dbh, TAC_DBINFO *dbinfo)
{
#if defined(DB_MYSQL)
	if (!mysql_init(dbh)) {
		if (debug & (DEBUG_AUTHEN_FLAG|DEBUG_ACCT_FLAG|DEBUG_PASSWD_FLAG))
			report(LOG_DEBUG, "db_connect: Cannot initialize");
		return (0);
	}
	if (!mysql_real_connect(dbh, dbinfo->host, dbinfo->user, dbinfo->password, dbinfo->database, 0, NULL, 0))
	{
		if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG|DEBUG_ACCT_FLAG))
			report(LOG_DEBUG, "db_connect: Cannot connect to %s database %s. Error: %s", dbinfo->host, dbinfo->database, mysql_error(dbh));

		/** Need add for redundancy here **/
		return (0);
	}
#endif

#if defined(DB_PGSQL)
	dbh->conn = PQsetdbLogin(dbinfo->host, NULL, NULL, NULL, dbinfo->database, dbinfo->user, dbinfo->password);
	if (PQstatus(dbh->conn) == CONNECTION_BAD) {
		if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG|DEBUG_ACCT_FLAG))
			report(LOG_DEBUG, "db_connect: Cannot connect to %s database %s. Error: %s", dbinfo->host, dbinfo->database, PQerrorMessage(dbh));
		return (0);
	}
#endif

	return (1);
}

void db_close(DBH *dbh)
{
#if defined(DB_MYSQL)
	mysql_close(dbh);
#endif

#if defined(DB_PGSQL)
	PQfinish(dbh->conn);
#endif
}

DB_RES *db_store_result(DBH *dbh)
{
#if defined(DB_MYSQL)
	return mysql_store_result(dbh);
#endif

#if defined(DB_PGSQL)
	return dbh->res;
#endif
}

int db_num_rows(DB_RES *result)
{
#if defined(DB_MYSQL)
	return mysql_num_rows(result);
#endif

#if defined(DB_PGSQL)
	return PQntuples(result);
#endif
}

DB_ROW db_fetch_row(DB_RES *result)
{
#if defined(DB_MYSQL)
	return mysql_fetch_row(result);
#endif
}

void db_free_result(DB_RES *result)
{
#if defined(DB_MYSQL)
	mysql_free_result(result);
#endif

#if defined(DB_PGSQL)
	PQclear(result);
#endif
}

int db_affected_rows(DBH *dbh)
{
#if defined(DB_MYSQL)
	return mysql_affected_rows(dbh);
#endif
}

#ifdef __STDC__
int db_query(DBH *dbh, char *querystmt,...)
#else
int db_query(dbh, querystmt, va_alist)
DBH *dbh;
char *querystmt;
va_dcl
#endif
{
	char *p, *sval, *sqlcmd, *bufp;
	char digits[16];
	int m, len = 0, ret = 1;
	va_list ap;

	bufp = sqlcmd = (char*)malloc(SQLCMDL);

#ifdef __STDC__
	va_start(ap, querystmt);
#else
	va_start(ap);
#endif

	for (p = querystmt; *p; p++) {
		if (*p != '%') {
			*bufp++ = *p;
			len++;
			continue;
		}
		switch (*++p) {
		case 's':
			sval = va_arg(ap, char *);
			m = strlen(sval);
			break;

		case 'u':
			sprintf(digits, "%u", va_arg(ap, uint));
			m = strlen(digits);
			sval = digits;
			break;

		case 'x':
			sprintf(digits, "%x", va_arg(ap, uint));
			m = strlen(digits);
			sval = digits;
			break;

		case 'd':
			sprintf(digits, "%d", va_arg(ap, int));
			m = strlen(digits);
			sval = digits;
			break;
		}

		memcpy(bufp, sval, m);
		bufp += m;
		len += m;
		continue;
	}

	sqlcmd[len] = '\0';
	va_end(ap);
	
#if defined(DB_MYSQL)
	if (mysql_real_query(dbh, sqlcmd, len)) {
		if (debug)
			report(LOG_DEBUG, "db_query: Cannot query using MySQL. Error - %s", mysql_error(dbh));
		ret = 0;
	}
#endif

#if defined(DB_PGSQL)
	dbh->res = PQexec(dbh->conn, sqlcmd);
	if (!dbh->res || PQresultStatus(dbh->res) != PGRES_COMMAND_OK )
	{
		if (debug)
			report(LOG_DEBUG, "db_query: Cannot query using PGSQL.  Error - %s", PQerrorMessage(dbh->conn));
		ret = 0;
	}
#endif

	free(sqlcmd);
	return(ret);
}

int
db_access( AUTHEN_DATA *data )
{
	int ret = 0;
	char *uid, *nas, *client_ip, *terminal, *status, *service;

	DBH dbh;
	DB_RES *res;
	DB_ROW row;

	if (!data) return ret;

	/** Point default db to the correct table for logging **/
	if ((uid=data->NAS_id->username) == NULL) {
		uid = "unknown";
	}

	if ((nas=data->NAS_id->NAS_name) == NULL) {
		nas = "unknown";
	}

	if ((client_ip=data->NAS_id->NAC_address) == NULL) {
		client_ip = "unknown";
	}

	if ((terminal=data->NAS_id->NAS_port) == NULL) {
		terminal = "unknown";
	}

	status = (data->status == TAC_PLUS_AUTHEN_STATUS_PASS) ? "accepted" : "rejected";

	switch (data->service) {
		case TAC_PLUS_AUTHEN_SVC_LOGIN:
			service = "login";
			break;
		case TAC_PLUS_AUTHEN_SVC_ENABLE:
			service = "enable";
			break;
		case TAC_PLUS_AUTHEN_SVC_PPP:
			service = "ppp";
			break;
		case TAC_PLUS_AUTHEN_SVC_ARAP:
			service = "arap";
			break;
		case TAC_PLUS_AUTHEN_SVC_PT:
			service = "pt";
			break;
		case TAC_PLUS_AUTHEN_SVC_RCMD:
			service = "rcmd";
			break;
		case TAC_PLUS_AUTHEN_SVC_X25:
			service = "x25";
			break;
		case TAC_PLUS_AUTHEN_SVC_NASI:
			service = "nasi";
			break;
		default:
			service = "unknown";
	}

	if (db_connect(&dbh, session.db_default)) {
		if (db_query(&dbh, ACCESSSQL, nas, terminal, uid, client_ip, service, status)) {
			if (db_affected_rows(&dbh)>0) {
				ret = 1;

				if (debug & DEBUG_ACCT_FLAG)
					report(LOG_DEBUG, "db_access: inserting record is successfull");
			}
		}
	}

	db_close(&dbh);
	return (ret);
}

int
db_acct( struct acct_rec *rec, TAC_DBINFO *dbinfo )
{
    int ret = 0;
    char *uid, *nas, *client_ip, *terminal, *type, *service, *priv_lvl, *cmd,
	 *elapsed_time, *bytes_in, *bytes_out;
    char *pUnknown = "unknown";

	DBH dbh;
	DB_RES *res;
	DB_ROW row;

	if (!dbinfo) {
		report(LOG_ERR, "db_acct: called with NULL db_info");
		return(0);
	}

	if (debug & DEBUG_ACCT_FLAG) 
		report(LOG_DEBUG,"db_acct: log accounting record to database");

    /* Check for empty passwords */
	if (dbinfo->user && dbinfo->password ) {
		if (!*(dbinfo->user) || !*(dbinfo->password)) {
			if (debug & DEBUG_ACCT_FLAG)
				report(LOG_ERR, "db_acct: Empty database userid or password");
			return (0);
		}
    } else {
		if (debug & DEBUG_ACCT_FLAG)
			report(LOG_DEBUG, "db_acct: Missing database userid or password");
		return (0);
	}

/* Find some attributes  for accounting */
    if ((uid=rec->identity->username) == NULL) {
        if (debug & DEBUG_ACCT_FLAG) 
			report(LOG_DEBUG,"db_acct: Can't find username!");
		uid = pUnknown;
	}
	if ((nas=rec->identity->NAS_name)==NULL) {
		if (debug & DEBUG_ACCT_FLAG) 
			report(LOG_DEBUG,"db_acct: Can't find NAS name!");
		nas = pUnknown;
	}
    if ((client_ip=rec->identity->NAC_address)==NULL) {
        	if (debug & DEBUG_ACCT_FLAG) 
			report(LOG_DEBUG,"db_acct: Can't find client address!");
		client_ip = pUnknown;
	}
    if ((terminal=rec->identity->NAS_port)==NULL) {
        	if (debug & DEBUG_ACCT_FLAG) 
			report(LOG_DEBUG,"db_acct: Can't find terminal!");
		terminal = pUnknown;
	}
    switch(rec->acct_type) {
    case ACCT_TYPE_UPDATE:
        type="update";
        break;
    case ACCT_TYPE_START:
        type="start";
        break;
    case ACCT_TYPE_STOP:
        type="stop";
        break;
    default:
        type=pUnknown;
        break;
    }
    if ((service=find_attr_value("service", rec->args, rec->num_args))==NULL) {
		service="NULL";
	}
    if ((priv_lvl=find_attr_value("priv-lvl", rec->args, rec->num_args))==NULL) {
		priv_lvl="NULL";
	}
    if ((elapsed_time=find_attr_value("elapsed_time", rec->args, rec->num_args))==NULL) {
		elapsed_time="NULL";
	}
    if ((bytes_in=find_attr_value("bytes_in", rec->args, rec->num_args))==NULL)
    	bytes_in="NULL";
    if ((bytes_out=find_attr_value("bytes_out",rec->args, rec->num_args))==NULL)
    	bytes_out="NULL";
    if ((cmd=find_attr_value("cmd", rec->args, rec->num_args))==NULL)
	cmd="";

	if (db_connect(&dbh, dbinfo)) {
		if (db_query(&dbh, ACCTSQL, session.db_default?"accounting":dbinfo->table, nas, uid, terminal, client_ip, type, service, priv_lvl, elapsed_time, bytes_in, bytes_out, cmd)) {
			if (db_affected_rows(&dbh)>0) {
				ret = 1;

				if (debug & DEBUG_ACCT_FLAG)
					report(LOG_DEBUG, "db_acct: inserting record is successfull");
			}
		}
	}

	/** Need to consolidate this information in the main table --ARY **/
#ifdef SHELL_ACCOUNT
    	if ((!strcasecmp(service,"shell"))&&(!session.db_default))
		ret = mysql_db_shell_acct(dbinfo, nas, client_ip, uid, rec);
#endif
    db_close(&dbh);

    if (debug)
	report(LOG_DEBUG, "db_acct: finish accounting");
    return (ret);

}

#endif /* DB */

