#include "tac_plus.h"

#if defined(USE_DB)
#include <stdio.h>

#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif


int db_get_host(char *field, tac_uint32 nas, char *data, TAC_DBINFO *dbinfo)
{
	int ret = 1;
	DBH dbh;
	DB_RES *res, *res1;
	DB_ROW row, row1;
	
	if (!field || !data || !dbinfo) {  /* no sense in continuing if we have NULL input */
		return(2);
	}

	if (debug)
		report(LOG_DEBUG, "db_get_host: getting %s from nas(%s)", field, 
			   cfg_addr_as_string(nas));

	if (db_connect(&dbh, dbinfo)) {
		if (db_query(&dbh, HOSTAUTHSQL, field, nas)) {
			res = db_store_result(&dbh);
			if (db_num_rows(res) >= 1) {
				while (ret && (row = db_fetch_row(res))) {
					if (tac_empty_string(row[0])) {
						if (db_query(&dbh, HOSTGROUPSQL, field, row[1])) {
							res1 = db_store_result(&dbh);
							if (db_num_rows(res1) > 0) {
								while (ret && (row1 = db_fetch_row(res1))) {
									if (tac_empty_string(row1[0])) {
										if (debug) report(LOG_DEBUG, "db_get_host: hostgroup is empty");
									} else {
										strcpy(data, row1[0]);
										if (debug) report(LOG_DEBUG, "db_get_host: got (%s) from hostgroup (%s)", data, row[1]);
										ret = 0;
									}
								}
								db_free_result(res1);
							}
						}
					} else {
						strcpy(data, row[0]);
						if (debug)
							report(LOG_DEBUG, "db_get_host: got (%s)", data);
						ret = 0;
					}
				}
			}
			db_free_result(res);
		}
		db_close(&dbh);
	}
	return ret;
}

int
db_verify(char *user, char *users_passwd, struct authen_data *data, int auth_type, int recurse )
{
   int cont=1, i, login_type=1;
   char member[30], password[40];
   DBH dbh;
   DB_RES *res;
   DB_ROW row;

   TAC_DBINFO *dbinfo = session.db_default;

   if (!dbinfo) {
	   report(LOG_ERR, "db_verify: called with NULL db_info");
	   return(0);
   }

   /* Check for empty userid or password */
   if (user && users_passwd) {
	   if (!*user || !*users_passwd) {
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			  report(LOG_DEBUG, "db_verify: called with empty userid or password");
		   return(0);
	   }
   } else {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		  report(LOG_DEBUG, "db_verify: called with missing userid or password");
	   return(0);
   }

   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
	   report(LOG_DEBUG, "db_verify: verify user %s to %s database", 
			  user, dbinfo->type);

   /* Check for empty passwords */
   if (dbinfo->user && dbinfo->password ) {
	   if (!*(dbinfo->user) || !*(dbinfo->password)) {
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_ERR, "db_verify: Empty database userid or password");
		   return (0);
	   }
   } else {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: Missing database userid or password");
	   return (0);
   }

   if (!db_connect(&dbh, dbinfo)) {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify:  Cannot connected to the databases");
	   return (0);
   }

   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
	   report(LOG_DEBUG,"db_verify: verify user(%s)", user);

   if (auth_type == S_enable) {
	   dbinfo->field_passwd = "enable";
	   login_type=0;
   } else {
	   dbinfo->field_passwd = "password";
   }

   if (!db_query(&dbh, AUTHSQL, dbinfo->field_passwd, dbinfo->table, dbinfo->field_name, user)) {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: database problems");
	   db_close(&dbh);
	   return(0);
   }

   res = db_store_result(&dbh);
   if (db_num_rows(res) == 0) {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: user(%s) not found", user);
	   db_close(&dbh);
	   return(0);
   }
   row = db_fetch_row(res);
   if (atoi(row[2])) {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: user(%s) is disabled", user);
	   db_free_result(res);
	   db_close(&dbh);
	   return (0);
   }

   strcpy(password, row[0]);
   strcpy(member, row[1]);
   i = atoi(row[3]);
   db_free_result(res);

   /** Implemented to support external authentication **/
   switch(ext_authen(user, users_passwd, i)) {
	   case 1:
		   goto profile;
		   break;
	   case 0:
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_DEBUG, "db_verify: external authentication failed");
		   return (0);
	   case 4:
	   case 5:
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_DEBUG, "db_verify: external communication problems");
		   return (0);
	   default:
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_DEBUG, "db_verify: external authentication failed");
		   break;
   }
   
   if (tac_empty_string(password)) {
   	if (!recurse) {
	  if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: recurse was not requested.");
	  db_close(&dbh);
	  return(0);
   	}

   	if (tac_empty_string(member)) {
	  if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: User(%s) is not a member of any group.", user);
	  db_close(&dbh);
	  return(0);
   	}

   	if (!db_query(&dbh, AUTHSQL, dbinfo->field_passwd, dbinfo->table, dbinfo->field_name, member)) {
	  db_close(&dbh);
	  return(0);
   	}

   	res = db_store_result(&dbh);
   	if (db_num_rows(res) == 0) {
	  if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: member(%s) not found", member);
	  db_close(&dbh);
	  return(0);
   	}
   	row = db_fetch_row(res);
   	strcpy(password, row[0]);
   	db_free_result(res);

   	if (tac_empty_string(password)) {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: Member password is blank.  Not allowed.");
	  db_close(&dbh);
	  return(0);
   	}
   }
	/** Verify password **/
   if (!des_verify(users_passwd, password)) {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: Password not valid.");
	   db_close(&dbh);
	   return (0);
   }

   /** Verify if user is allowed to access the NAS **/
profile:
   if (session.peeraddr) {
	   if (db_get_profile(&dbh, user, member, login_type, &i, &i)) {
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_DEBUG, "db_verify: User(%s) or member(%s) is not authorized to use NAS(%s)", user, member, session.peer);
		   db_close(&dbh);
		   return (0);
	   }
   }

   /** Verify if user is allowed to access the NAS from current IP **/
   if (!tac_empty_string(data->NAS_id->NAC_address)) {
	   if (!db_query(&dbh, USERACCESSSQL, user, member, data->NAS_id->NAC_address)) {
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_DEBUG, "db_verify: database problems.");
		   db_close(&dbh);
		   return(0);
	   }
	   res = db_store_result(&dbh);
	   if (db_num_rows(res)) {
		   cont = 1;
		   while (cont && (row = db_fetch_row(res))) {
			   if (strcmp(row[0],"58")==0) {
				   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
					   report(LOG_DEBUG,"db_verify: User is not authorized to access nas(%s) from this address(%s)", session.peer, data->NAS_id->NAC_address);
				   db_free_result(res);
				   db_close(&dbh);
				   return(0);
			   } else if (strcmp(row[0],"57")==0)
				   cont=0;
		   }
		   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
			   report(LOG_DEBUG, "db_verify: User is authorized to access nas(%s) from this address(%s)", session.peer, data->NAS_id->NAC_address);
	   }
   } else {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_verify: User is login in through console");
   }

	db_close(&dbh);
	return (1); 
}

int db_author( struct author_data *data )
{
	int ret = 1, svc;
	DBH dbh;
	DB_RES *res;
	DB_ROW row;
	char *cmd, *protocol, *svcname;
	struct author_user user;

	data->status = AUTHOR_STATUS_FAIL;
	data->output_args = NULL;
	data->num_out_args = 0;

	if (debug & DEBUG_AUTHOR_FLAG)
		report(LOG_DEBUG, "db_author: authorizing user(%s) on nas(%s)", data->id->username, session.peer);

	strcpy(user.username, data->id->username);
	protocol = NULL;

	if (db_connect(&dbh, session.db_default)) {
		if (db_query(&dbh, "SELECT b_author, a_author, svc_dflt, cmd_dflt, gid FROM %s where uid = '%s'", session.db_default->table, user.username)) {
			res = db_store_result(&dbh);
			if (db_num_rows(res) > 0) {
				row = db_fetch_row(res);
				strcpy(user.pre_author, row[0]?row[0]:"");
				strcpy(user.post_author, row[1]?row[1]:"");
				user.svc_dflt = atoi(row[2]?row[2]:"0");
				user.cmd_dflt = atoi(row[3]?row[3]:"0");
				strcpy(user.member, row[4]?row[4]:"");
				db_free_result(res);

				if (!db_get_profile(&dbh, user.username, user.member, 2, &user.profile, &user.vid)) {
					if (user.profile && db_query(&dbh, "SELECT uid FROM user WHERE id=%u AND user=3", user.profile)) {
						res = db_store_result(&dbh);
						if (db_num_rows(res)>0) {
							row = db_fetch_row(res);
							strcpy(user.username, row[0]);
							strcpy(user.member, "");
							db_free_result(res);
						}
					}
				}

				if ( pre_authorization(user.username, user.pre_author[0]?user.pre_author:NULL, data)) {
					db_close(&dbh);
					return (0);
				}

				if (!(svc=get_nas_svc(data, &cmd, &protocol, &svcname))) {
					data->status = AUTHOR_STATUS_ERROR;
					data->admin_msg = tac_strdup("No identifiable service/protocol in authorization request");
					if (debug & DEBUG_AUTHOR_FLAG)
						report(LOG_DEBUG, "db_author: %s for %s", data->admin_msg, user.username);
					db_close(&dbh);
					return (0);
				}

				switch (svc) {
					default:
						report(LOG_ERR, "db_author: bad service type (%d) from %s", svc, session.peer);
						break;

					case N_svc_cmd:
						ret = db_authorize_cmd(&dbh, &user, cmd, data);
						break;

					case N_svc_exec:
						if (db_authorize_exec(&dbh, &user, data))
								return (0);
						
					case N_svc_arap:
					case N_svc_ppp:
					case N_svc_slip:
						ret = db_authorize_svc(&dbh, &user, svc, protocol, NULL, data);
						break;

					case N_svc:
						ret = db_authorize_svc(&dbh, &user, svc, protocol, svcname, data);
						break;
				}

				post_authorization(user.username, user.post_author[0]?user.post_author:NULL, data);
			}
		}
		db_close(&dbh);
	} else ret = 0;

	if (debug & DEBUG_AUTHOR_FLAG)
		report(LOG_DEBUG, "db_author: finish authorization");
	return (ret);
}

static char ** get_svc_attrs(DB_RES *res, int *denyp)
{
	DB_ROW row;
	char **args;
	int i = db_num_rows(res);

	*denyp = 1;
	args = (char**) tac_malloc(sizeof(char *) * (i+1));
	i = 0;
	while (row = db_fetch_row(res)) {
		char *arg = tac_strdup(row[1]);
		char *p = index(arg, '=');

		if (p && atoi(row[0]) == N_optarg)
			*p = '*';
		args[i++] = arg;
	}
	args[i] = NULL;
	return (args);
}

int get_svc_node(DBH *dbh, char *user, int vendor, int svc, char *value, char *value1)
{
//	DB_RES *res;
//	DB_ROW row;
	char *_vid;
	int ret = 0;

	if (debug & DEBUG_AUTHOR_FLAG) {
		report(LOG_DEBUG, "get_svc_node: user (%s) service(%d) vendor(%d)", user, svc, vendor);
	}

	/* Check to see if the device was assigned a vendor */
	_vid = (char*)tac_malloc(30);
	if (vendor) {
		sprintf(_vid,"AND vid = %d", vendor);
	} else {
		sprintf(_vid,"");
	}

	switch (svc) {
		case N_svc:
			if (!db_query(dbh, "SELECT type, value FROM node WHERE uid = '%s' AND service = %d AND value1 ='%s' %s", user, svc, value1, _vid))
				ret = 1;
			break;

		case N_svc_ppp:
			if (!db_query(dbh, "SELECT type, value FROM node WHERE id = '%s' AND service = %d AND value1 = '%s' %s", user, svc, value1, _vid) )
				ret = 1;
			break;
			
	        case N_svc_cmd:
		       if (!db_query(dbh, "SELECT type, value1 FROM node WHERE uid = '%s' AND service = %d AND value = '%s' AND '%s' REGEXP value1 %s ORDER BY seq", user, svc, value, value1, _vid)) 
			       ret = 1;
		       break;
		case N_svc_exec:
		       if (!db_query(dbh, "SELECT type, CONCAT(value1,'=',value) AS value FROM node WHERE uid = '%s' AND service = %d %s", user, svc, _vid))
			       ret = 1;
		       break;
		default:
		       if (!db_query(dbh, "SELECT type, value FROM node WHERE uid = '%s' AND service = %d %s", user, svc, _vid) )
		                ret = 1;
	}

	free(_vid);
	return (ret);
}

int db_authorize_svc(dbh, user, svc, protocol, svcname, data)
DBH *dbh;
struct author_user *user;
int svc;
char *protocol, *svcname;
struct author_data *data;
{
	DB_RES *res;
	char **cfg_args;
	int deny_by_default;

	if (debug & DEBUG_AUTHOR_FLAG) {
		report(LOG_DEBUG, "db_authorize_svc: service authorization for %s", user->username);
	}

	if (get_svc_node(dbh, user->username, user->vid, svc, protocol, svcname)) {
		data->status = AUTHOR_STATUS_ERROR;
		data->num_out_args = 0;
		return (1);
	}

	res = db_store_result(dbh);
	if (db_num_rows(res) <= 0) {
		db_free_result(res);

		if (!user->member) goto NEXT;
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "db_authorise_svc: service authorization for %s", user->member);

		if (get_svc_node(dbh, user->member, user->vid, svc, protocol, svcname)) {
			data->status = AUTHOR_STATUS_ERROR;
			data->num_out_args = 0;
			return (1);
		}

		res = db_store_result(dbh);
		if (db_num_rows(res) >= 1)
			goto NEXT2;
		db_free_result(res);
NEXT: 
		if (user->svc_dflt ==  1) {
			if (debug & DEBUG_AUTHOR_FLAG)
				report(LOG_DEBUG,"db_authorize_svc: svc=%s protocol=%s svcname=%s not found, permitted by default",
					cfg_nodestring(svc), protocol ? protocol : "",
					svcname ? svcname : ""); 

			data->status = AUTHOR_STATUS_PASS_ADD;
			data->num_out_args = 0;
			data->output_args = NULL;
			return (0);
		}

		if (debug & DEBUG_AUTHOR_FLAG)
				report(LOG_DEBUG,"db_authorize_svc: svc=%s protocol=%s not found, denied by default",
					cfg_nodestring(svc), protocol ? protocol : ""); 

		data->status = AUTHOR_STATUS_FAIL;
		data->output_args = NULL;
		data->num_out_args = 0;
		return (0);
	}
NEXT2:
	cfg_args = get_svc_attrs( res, &deny_by_default);
	return (author_svc(cfg_args, deny_by_default, data));
}

int db_authorize_cmd(dbh, user, cmd, data)
DBH *dbh;
struct author_user *user;
char *cmd;
struct author_data *data;
{
	DB_RES *res;
	DB_ROW row;
	char *args;
//	int cont = 1;

	data->num_out_args = 0;

	args = assemble_args(data);
	data->num_out_args = 0;

	if (debug & DEBUG_AUTHOR_FLAG)
		report(LOG_DEBUG, "db_authorize_cmd: command (%s %s) authorization for %s and %s", cmd?cmd:"", args?args:"", user->username, user->member);


	if (!cmd) {
		data->status = AUTHOR_STATUS_ERROR;
		data->admin_msg = tac_strdup("No command found");
		report(LOG_ERR, "db_authorize_cmd: %s - %s %s", session.peer, cmd, data->admin_msg);
		if (args) free(args);
		return (0);
	}

	if (get_svc_node(dbh, user->username, user->vid, N_svc_cmd, cmd, args?args:"")) {
		data->status = AUTHOR_STATUS_ERROR;
		if (args) free(args);
		return (1);
	}
	res = db_store_result(dbh);
	if (db_num_rows(res) <= 0) {
//		db_free_result(res);
		if (user->member) {
			if (debug & DEBUG_AUTHOR_FLAG)
				report(LOG_DEBUG, "db_authorize_cmd: command (%s %s) authorization for %s", cmd?cmd:"", args?args:"", user->member);
			if (get_svc_node(dbh, user->member, user->vid, N_svc_cmd, cmd, args?args:"")) {
				data->status = AUTHOR_STATUS_ERROR;
				if (args) free(args);
				return (1);
			}
			res = db_store_result(dbh);
			if (db_num_rows(res) >= 1)
				goto NEXT4;
		}

		if (args) free(args);

                if (user->cmd_dflt ==  1) {
                        if (debug & DEBUG_AUTHOR_FLAG)
                                report(LOG_DEBUG,"db_authorize_cmd: command(%s) not found, permitted by default", cmd);
                        data->status = AUTHOR_STATUS_PASS_ADD;
                        return (0);
                }

                if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG,"db_authorize_cmd: command(%s) not found, denied by default", cmd);

                data->status = AUTHOR_STATUS_FAIL;
                return (0);
	}

	/** command exists.  check if command is DENIED or PERMITTED **/
NEXT4:
	row = db_fetch_row(res);
	switch (atoi(row[0])) {
	case N_permit:
		if (debug & DEBUG_AUTHOR_FLAG) 
			report(LOG_DEBUG, "db_authorize_cmd: command(%s %s) permitted", cmd, args);
		data->status = AUTHOR_STATUS_PASS_ADD;
		break;
	case N_deny:
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "%s %s denied", cmd, args);
		data->status = AUTHOR_STATUS_FAIL;
		break;
	default:
		data->status = AUTHOR_STATUS_ERROR;
		data->admin_msg = tac_strdup("Server error illegal configuration");
		report(LOG_ERR, "%s: %s %s %s", session.peer, cmd, args,
			data->admin_msg);
		break;
	}

	if (args) free(args);
	db_free_result(res);
	return (0);
}

int
db_authorize_exec(dbh, user, data)
DBH *dbh;
struct author_user *user;
struct author_data *data;
{
	DB_RES *res;
	int num_rows;

	if (debug & DEBUG_AUTHOR_FLAG)
		report(LOG_DEBUG, "db_authorize_exec: exec authorization for %s member of %s", user->username, user->member);

//	if (user->member) {
		if (!db_query(dbh, "SELECT service FROM node WHERE uid IN ('%s','%s') and service IN (%d,%d)" , user->username, user->member, N_svc_exec, N_svc_cmd)) {
			data->status = AUTHOR_STATUS_ERROR;
			data->output_args = NULL;
			data->num_out_args = 0;
			return (1);
		}
/*	} else {
		if (!db_query(dbh, "SELECT service FROM node WHERE uid = '%s' and service IN (%d,%d)" , user->username, N_svc_exec, N_svc_cmd)) {
			data->status = AUTHOR_STATUS_ERROR;
			data->output_args = NULL;
			data->num_out_args = 0;
			return (1);
		}
	} */
	res = db_store_result(dbh);
	num_rows = db_num_rows(res);
	if (num_rows > 0) {
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "db_authorize_exec: exec is explicitly permitted");
		db_free_result(res);
		return (0);
	}
	db_free_result(res);
/*
	if  (user->member) {
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "db_authorize_exec: %s is part of a group. Authorizing user and group for commands.", data->id->username);
		if (!db_query(dbh, "SELECT service FROM node WHERE uid IN ('%s', '%s') and service = %d" , user->username, user->member, N_svc_cmd)) {
			data->status = AUTHOR_STATUS_ERROR;
			data->output_args = NULL;
			data->num_out_args = 0;
			return (1);
		}
	} else {
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "db_authorize_exec: %s is not part of a group. Authorizing user for commands.", data->id->username);
		if (!db_query(dbh, "SELECT service FROM node WHERE uid = '%s' and service = %d" , user->username, N_svc_cmd)) {
			data->status = AUTHOR_STATUS_ERROR;
			data->output_args = NULL;
			data->num_out_args = 0;
			return (1);
		}
	}
	res = db_store_result(dbh);
	if (db_num_rows(res) > 0) {
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "db_authorize_exec: commands configured thus exec permitted");
		data->status = AUTHOR_STATUS_PASS_ADD;
		data->output_args = NULL;
		data->num_out_args = 0;
		return (1);
	}
	db_free_result(res);
*/
	if (user->svc_dflt == 1) {
		if (debug & DEBUG_AUTHOR_FLAG)
			report(LOG_DEBUG, "db_author: exec permitted by default");
		data->status = AUTHOR_STATUS_PASS_ADD;
		data->output_args = NULL;
		data->num_out_args = 0;
		return(1);
	}

	if (debug & DEBUG_AUTHOR_FLAG)
		report(LOG_DEBUG, "db_authorize_exec: exec denied by default");

	data->status = AUTHOR_STATUS_FAIL;
	data->output_args = NULL;
	data->num_out_args = 0;
	return (1);
}

int
db_get_profile(DBH *dbh, char *user, char *member, int logintype, tac_uint32 *profile, int *vendor)
{
	int num_rows, i, acl[15], cont=1;
	char hostgroup[20]="";
	DB_RES *res, *res1;
	DB_ROW row, row1;

	if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG|DEBUG_AUTHOR_FLAG))
		report(LOG_DEBUG, "db_get_profile: authorizing user (%s) member of (%s)", user, member);

   	if (db_query(dbh, HOSTACLSQL,logintype?"loginacl":"enableacl", session.peeraddr)) {
		res = db_store_result(dbh);
		if ((num_rows=db_num_rows(res)) == 0) {
			if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
				report(LOG_DEBUG, "db_get_profile: NAS(%s) is not defined.", session.peer);
			db_free_result(res);
			return (cont);
		}
		for (i=0;row=db_fetch_row(res);i++) {
			acl[i] = atoi(row[0]);
			if (i==0) *vendor=atoi(row[2]);
			if (!tac_empty_string(hostgroup)&&strcmp(hostgroup,row[1]?row[1]:"")) {
				if (db_query(dbh, HOSTGROUPSQL, logintype?"loginacl":"enableacl", hostgroup)) {
					res1 = db_store_result(dbh);
					if (db_num_rows(res1) > 0) {
						row1=db_fetch_row(res1);
						i++; num_rows++;
						acl[i] = atoi(row1[0]);
					}
					db_free_result(res1);
				}
			}
			strcpy(hostgroup, row[1]?row[1]:"");
		}
		db_free_result(res);
		if (!tac_empty_string(hostgroup)) {
			if (db_query(dbh, HOSTGROUPSQL, logintype?"loginacl":"enableacl", hostgroup)) {
				res = db_store_result(dbh);
				if (db_num_rows(res) > 0) {
					row=db_fetch_row(res);
					acl[num_rows] = atoi(row[0]);
					num_rows++;
				}
				db_free_result(res);
			}
		}
		for (i=0;cont && (i<num_rows);i++) {
			if (acl[i]) {
				if (db_query(dbh, NASACLSQL, acl[i], user, member)) {
					res = db_store_result(dbh);
					if (db_num_rows(res) > 0) {
						while (cont && (row=db_fetch_row(res))) {
							if (strcmp(row[0],"58")==0) {
								if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
									report(LOG_DEBUG, "db_get_profile: user(%s) is not allowed to access the nas(%u).");
								db_free_result(res);
								return (cont);
							} else if (strcmp(row[0],"57")==0)  {
								cont = 0;
								*profile = atol(row[1]);
							}
						}
					}
					db_free_result(res);
				} else {
					if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
						report(LOG_DEBUG, "db_get_profile: problems with NASACLSQL query");
				}
			}
		}
		if (cont) {
			if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
				report(LOG_DEBUG, "db_get_profile: user(%s) do not have permission to access this nas(%s)", user, session.peer);
		}
	} else {
	   if (debug & (DEBUG_PASSWD_FLAG|DEBUG_AUTHEN_FLAG))
		   report(LOG_DEBUG, "db_get_profile: problems with HOSTACL query");
	}
	return (cont);
}

#endif /* DB */

