/**
 * File:	ext_authen.c
**/
#include "ext_authen.h"

#ifndef FREEBSD
#define USE_ACE_AGENT_API_PROTOTYPES
#include "inc/acexport.h"

/**
 * Function:	SD_Authenticate
 * Parameters:	user - user id
 * 		user_passwd - user password
 * Returns:	0 - authentication failed
 * 		1 - authentication passed
 * 		4 - cannot communicate with AceServer
 * 		5 - cannot lock user name
**/
int SD_Authenticate( char *user, char *user_passwd) {
	int acmRet, ret = 0;
	SDI_HANDLE SdiHandle = SDI_HANDLE_NONE;

	acmRet = SD_Init(&SdiHandle);
	if (acmRet != ACM_OK)
		return (4);

	acmRet = SD_Lock(SdiHandle, user);
	if (acmRet != ACM_OK)
		return (5);

	acmRet = SD_Check(SdiHandle, user_passwd, user);
	switch (acmRet) {
		case ACM_OK:
			ret = 1;
			break;
			
		case ACM_INVALID_SERVER:
			ret = 4;
			break;
			
		case ACM_ACCESS_DENIED:
		case ACM_NEXT_CODE_REQUIRED:
		case ACM_NEW_PIN_REQUIRED:
			break;

		default:
			ret = 4;
	}

	SD_Close(SdiHandle);

	return (ret);
}
#else
int SD_Authenticate( char *user, char *user_passwd) {
	return (0);
}
#endif

/**
 * Function:	ext_authen
 * Parameters:	user - user id
 * 		user_passwd - user password
**/
int ext_authen( char *user, char *user_passwd, int auth )
{
	int ret = 0;

	switch (auth) {
		case 1: /** Check for local authentication **/
			ret = 2;
			break;
		case 5: /** SecureID authentication **/
			ret = SD_Authenticate(user, user_passwd);
			break;
	}

	return (ret);
}
